import hashlib
import logging
from contextlib import closing

import boto3
from django.conf import settings
from django.core.files.temp import NamedTemporaryFile, gettempdir

logger = logging.getLogger(__name__)

ENGINE = getattr(settings, "WAGTAIL_SPEECH_ENGINE", "standard")
LANGUAGE_CODE = getattr(settings, "WAGTAIL_SPEECH_LANGUAGE_CODE", "en-EN")
OUTPUT_FORMAT = getattr(settings, "WAGTAIL_SPEECH_OUTPUT_FORMAT", "mp3")
SAMPLE_RATE = getattr(settings, "WAGTAIL_SPEECH_SAMPLE_RATE", "8000")
VOICE_ID = getattr(settings, "WAGTAIL_SPEECH_VOICE_ID", "Joey")


def get_hash_from_tts_context(value):
    hash = hashlib.md5(value.encode()).hexdigest()
    return hash


def synthesize_speech(page, text):
    """
    Synthesize speech to an audio stream.
    """
    logger.info("Synthesize speech for %s" % page)
    text_type = "ssml" if text.startswith("<speak>") else "text"
    client = boto3.client("polly")

    res = client.synthesize_speech(
        Engine=ENGINE,
        LanguageCode=LANGUAGE_CODE,
        OutputFormat=OUTPUT_FORMAT,
        SampleRate=SAMPLE_RATE,
        Text=text,
        TextType=text_type,
        VoiceId=VOICE_ID,
    )
    if "AudioStream" in res:
        with closing(res["AudioStream"]) as stream:
            try:
                temp_dir = gettempdir()
                temp_file = NamedTemporaryFile(dir=temp_dir)
                temp_file.write(stream.read())
            except IOError as error:
                logger.error("An error occured: %s" % error)
        return temp_file
    else:
        logger.error(
            "No audio stream found in synthesize_speech response %s" % str(res)
        )


def get_tts_context_from_stream_field(request, field, context=None):
    """
    Get speech text from a StreamField.

    This methods iterates over the stream field blocks which have the method
    get_tts_context implemented. It concats these values into a compatible
    multiline string value.
    """
    values = []
    for child in field:
        if hasattr(child.block, "get_tts_context"):
            values.append(child.block.get_tts_context(child.value))
    text = '<break strength="x-strong"/>'.join(values)
    return "<speak>%s</speak>" % text



# Generated by Django 5.0.9 on 2024-11-25 12:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0093_uploadedfile"),
    ]

    operations = [
        migrations.CreateModel(
            name="TextToSpeechEntry",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("hash", models.TextField()),
                (
                    "audio_stream",
                    models.FileField(upload_to="", verbose_name="audio stream"),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, null=True, verbose_name="created at"
                    ),
                ),
                (
                    "page",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtailcore.page",
                    ),
                ),
            ],
            options={
                "ordering": ["-created_at"],
            },
        ),
    ]



from django.contrib import admin

from wagtailspeech.models import TextToSpeechEntry


class TextToSpeechEntryAdmin(admin.ModelAdmin):
    list_display = ("page", "created_at")
    list_select_related = ("page",)


admin.site.register(TextToSpeechEntry, TextToSpeechEntryAdmin)



from django.urls import path

from wagtailspeech.views import SynthesizeSpeechView

app_name = "wagtailspeech"

urlpatterns = [
    path(
        "synthesize-speech/<str:id>/",
        SynthesizeSpeechView.as_view(),
        name="synthesize-speech",
    ),
]  # noqa



from django.db import models
from django.utils.translation import gettext_lazy as _
from wagtail.models import Page


class TextToSpeechMixin(models.Model):
    """
    Mixin class to support synthesize speech functionalities.

    Example:
        class HomePage(TextToSpeechMixin, Page):
            def get_tts_context(self, request):
                return self.title
    """

    class Meta:
        abstract = True

    def get_tts_context(self, request):
        return None


class TextToSpeechEntry(models.Model):
    page = models.ForeignKey(
        "wagtailcore.Page",
        on_delete=models.CASCADE,
        related_name="+",
    )
    hash = models.TextField()
    audio_stream = models.FileField(_("audio stream"))
    created_at = models.DateTimeField(
        verbose_name=_("created at"), auto_now_add=True, null=True
    )

    class Meta:
        ordering = ["-created_at"]

    def __str__(self):
        return str(self.page)



from django.conf import settings
from django.core.files import File
from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from django.views import View
from wagtail.models import Page

from wagtailspeech.models import TextToSpeechEntry, TextToSpeechMixin
from wagtailspeech.utils import get_hash_from_tts_context, synthesize_speech

OUTPUT_FORMAT = getattr(settings, "WAGTAIL_SPEECH_OUTPUT_FORMAT", "mp3")


class SynthesizeSpeechView(View):
    def get(self, request, id, *args, **kwargs):
        try:
            page = Page.objects.get(id=id)
        except Page.DoesNotExist:
            res = JsonResponse({"error": "Page not found"})
            res.status_code = 404
            return res

        specific_page = page.specific

        if not isinstance(specific_page, TextToSpeechMixin):
            res = JsonResponse({"error": "Page is not instance of TextToSpeechMixin"})
            res.status_code = 500
            return res

        tts_context = specific_page.get_tts_context(request)
        tts_hash = get_hash_from_tts_context(tts_context)

        try:
            entry = TextToSpeechEntry.objects.get(page=page, hash=tts_hash)
        except TextToSpeechEntry.DoesNotExist:
            audio_stream = synthesize_speech(page, tts_context)

            entry = TextToSpeechEntry.objects.create(
                page=page,
                hash=tts_hash,
                audio_stream=File(
                    audio_stream,
                    name="%s.%s.%s"
                    % (
                        page.id,
                        tts_hash,
                        OUTPUT_FORMAT,
                    ),
                ),
            )

        return JsonResponse(
            {
                "text": tts_context,
                "audio_stream": entry.audio_stream.url,
            }
        )
